@@ -12,9 +12,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import json
-
 from oslo_log import log as logging
+from oslo_serialization import jsonutils as json
 
 from tempest_lib.common import rest_client
 from tempest_lib import exceptions
@@ -85,16 +84,17 @@ class TokenClient(rest_client.RestClient):
 
         resp, resp_body = self.raw_request(url, method,
                                            headers=headers, body=body)
-        self._log_request(method, url, resp)
+        self._log_request(method, url, resp, req_headers=headers,
+                          req_body='<omitted>', resp_body=resp_body)
 
         if resp.status in [401, 403]:
-            resp_body = self._json_loads(resp_body)
+            resp_body = json.loads(resp_body)
             raise exceptions.Unauthorized(resp_body['error']['message'])
         elif resp.status not in [200, 201]:
             raise exceptions.IdentityError(
                 'Unexpected status code {0}'.format(resp.status))
 
-        return resp, self._json_loads(resp_body)
+        return resp, json.loads(resp_body)
 
     def get_token(self, user, password, tenant, auth_data=False):
         """Returns (token id, token data) for supplied credentials."""
